import datetime
from flask_login import UserMixin


class WorkUnit():
    """A unit of work.
    start   : datetime  -> when the work was started
    end     : datetime  -> when the work was ended
    length  : timedelta -> hours spent doing work
    """
    def __init__(self, start, end, length=None):
        self.start = start
        self.end = end
        self.length = end - start

    def get_break_time(self):
        """An employee gets 30 minutes unpaid break time
        every 5 hours. Result is in minutes."""
        FIVE_HOURS = (60*60*5)
        if self.length.seconds > FIVE_HOURS:
            num_breaks = (self.length.seconds // FIVE_HOURS)
            return num_breaks * 30
        else:
            return 0

    def get_length(self):
        """Returns the length of the work unit, ignoring breaks, in minutes"""
        return self.length.seconds / 60

    def get_billable_length(self):
        """Returns the length of the work unit, minus breaks, in minutes"""
        return self.get_length() - self.get_break_time()

    def weekday(self):
        """Number day of the week starting on Saturday = 0"""
        return (self.start.weekday() + 2) % 7


class PayPeriod():
    """A two week pay period
    employee    : Employee      -> Employee object, who did the work
    start_date  : date          -> the first day of the pay period
    work_units  : [WorkUnit]    -> collection, all the work in a week
    """
    def __init__(self, employee, start_date, work_units=[], complete=False,
                    payrate=None):
        self.employee = employee
        self.start_date = start_date
        self.work_units = work_units
        self.payrate = payrate or employee.payrate
        self.complete = complete

    def end_date(self):
        """Date that a pay period ends (one day before 2 weeks)"""
        return self.start_date + datetime.timedelta(weeks=1, days=6)

    def next_pay_period(self):
        """Date of next pay period (2 weeks)"""
        return self.start_date + datetime.timedelta(weeks=2)

    def get_total_shifts(self):
        return len(self.work_units)

    def get_total_hours(self):
        total_minutes = 0
        for unit in self.work_units:
            total_minutes += unit.get_length()
        return total_minutes / 60

    def get_total_break_hours(self):
        total_minutes = 0
        for unit in self.work_units:
            total_minutes += unit.get_break_time()
        return total_minutes / 60

    def get_total_billable_hours(self):
        total_minutes = 0
        for unit in self.work_units:
            total_minutes += unit.get_billable_length()
        return total_minutes / 60

    def get_total_payroll_amount(self):
        return self.payrate * self.get_total_billable_hours()

    def week_totals(self):
        """A tuple containing the total hours worked for each week"""
        total_week1 = datetime.timedelta()
        total_week2 = datetime.timedelta()
        for work_unit in self.work_units:
            if is_first_week(self.start_date, work_unit.start):
                total_week1 += work_unit.length
            else:
                total_week2 += work_unit.length
        return total_week1.total_seconds()/3600, total_week2.total_seconds()/3600


class User(UserMixin):
    """A User object used for logging in."""
    def __init__(self, netid, fullname="", type="", descduty="",
                 payrate=0.0, account_num=0, email=""):
        self.netid = netid
        self.fullname = fullname
        self.type = type
        self.descduty = descduty
        self.payrate = payrate
        self.id = netid
        self.account_num = account_num
        self.email = email


class Employee():
    """Represents an employee for ease of data manipulation.
    netid       : string    -> Employee's netID. This is his/her identity.
    fullname    : string    -> Name of employee
    type        : string    -> Employee type
    description : string    -> Description of duty
    payrate     : float     -> Hourly wage
    email       : string    -> Email address
    account_num : integer   -> Account number
    """
    def __init__(self, netid, fullname="", type="", description="",
                 payrate=0.0, account_num=0, email=""):
        self.netid = netid
        self.fullname = fullname
        self.type = type
        self.description = description
        self.payrate = payrate
        self.account_num = account_num
        self.email = email


def is_first_week(start_date, checked_date):
    """Determine which week in the pay period a date is"""
    start_date = datetime.datetime.combine(start_date, datetime.datetime.min.time())
    return checked_date < (start_date + datetime.timedelta(weeks=1))
