"""Form classes for views.py"""

import datetime
from flask.ext.wtf import Form
from flask_auth import LoginForm
from util import (get_start_date, get_end_date, previous_pp,
                  DAY_FORMAT, PP_FORMAT)
from wtforms import (SelectField, FieldList, FormField, TextField,
                     SubmitField, IntegerField, DecimalField, HiddenField,
                     BooleanField, DateField, PasswordField)
from wtforms.validators import Email, DataRequired, AnyOf, NumberRange


class PeriodForm(Form):
    """Form for selecting a pay period to view"""
    def __init__(self, **kwargs):
        """Call the parent __init__ and create dropdown choices"""
        super(PeriodForm, self).__init__(**kwargs)
        today_start_date = get_start_date()
        today_end_date = get_end_date(today_start_date)
        choices = []
        for x in range(0, 5):
            start_date = previous_pp(today_start_date, x)
            start_date_str = start_date.strftime(PP_FORMAT)
            if x == 0:
                start_date_str = "**" + start_date_str

            end_date = previous_pp(today_end_date, x)
            end_date_str = end_date.strftime(PP_FORMAT)
            if x == 0:
                end_date_str += "**"

            choices.append((start_date.strftime(DAY_FORMAT),
                            "{} - {}".format(start_date_str, end_date_str)))
        self.period.choices = choices

    period = SelectField('Pay Period')


class DateRangeForm(Form):
    """Form for selecting a date range for viewing statistics"""
    start_date = DateField('Start date')
    end_date = DateField('End date')


class Workday(Form):
    """Form which represents a single work day"""
    def __init__(self, **kwargs):
        """Call the parent __init__ and create dropdown choices"""
        super(Workday, self).__init__(**kwargs)

        choices = [("--", "--:--")]
        for h in range(24):
            t = datetime.time(hour=h, minute=0)
            choices.append((t.strftime("%H%M"), t.strftime("%I:%M %p")))
            t = datetime.time(hour=h, minute=30)
            choices.append((t.strftime("%H%M"), t.strftime("%I:%M %p")))
       
        self.start_time.choices = choices
        self.end_time.choices = choices

    def set_start_default(self, _id):
        self.start_time.default = _id
        self.process()

    def set_end_default(self, _id):
        self.end_time.default = _id
        self.process()

    start_time = SelectField('Start time')
    end_time = SelectField('End time')
    date_field = HiddenField()


class WorkWeek(Form):
    """Form which represents one week in a pay period"""
    def __init__(self, start_date, **kwargs):
        """Call the parent __init__ and add dates to Workday objects"""
        super(WorkWeek, self).__init__(**kwargs)
        self.start_date = start_date
        for i in range(14): #TODO: Change to 7
            this_date = start_date + datetime.timedelta(days=i)
            self.days.entries[i].date = this_date
            self.days.entries[i].date_field.data = str(this_date)

    days = FieldList(FormField(Workday), min_entries=14, max_entries=14)


class HoursForm(Form):
    """Form which represents a pay period"""
    def __init__(self, start_date=datetime.date.today(), **kwargs):
        """Call the parent __init__ and create WorkWeek objects"""
        super(HoursForm, self).__init__(**kwargs)
        self.start_date = start_date

        week1 = WorkWeek(start_date)
        #week2 = WorkWeek2(start_date + datetime.timedelta(weeks=1))
        self.weeks = [week1] # , week2]

    def checkmark_complete(self):
        self.complete.default = True
        self.process()

    submit_hours = SubmitField()
    complete = BooleanField()


class EmployeeForm(Form):
    """Initialize all necessary fields for an employee form"""
    netid = TextField('netid',
        validators=[DataRequired("Please enter your NetID.")])
    name = TextField('name',
        validators=[DataRequired("Please enter your full name.")])
    employee_type = SelectField('employee_type', coerce=int,
        choices=[
            (4, 'Type 4: Casual hourly / seasonal part-time'), 
            (5, 'Type 5: Hourly student'), 
            (6, 'Type 6: Salaried student'), 
            (7, 'Type 7: TA / part-time lecturer')
        ],
    )
    title = TextField(
        'title',
        validators=[DataRequired("Please enter your job title.")])
    hourly_wage = DecimalField(
        'hourly_wage',
        validators=[
            NumberRange(message="Please enter a valid wage.",
                        min=8.38, max=20.00),
            DataRequired("Please enter your hourly wage.")
        ]
    )
    account_num = SelectField('account_num', coerce=int,
        choices=[
            (202823, 'Enterprise Messaging (202823)'), 
            (289243, 'EM Mobile Apps (289243)')
        ]
    )
    email = TextField(
        'email',
        validators=[
            Email(),
            DataRequired("Please enter a valid email address.")
        ]
    )
    submit_button = SubmitField()


class RULoginForm(LoginForm):
    """A WTForm representing the login form."""
    username = TextField('Netid', validators=[DataRequired()])
    password = PasswordField('Password',
                             validators=[DataRequired()])


def prepopulate(form, current_user):
    """ Prepopulate an EmployeeForm with available current_user information """
    form.netid.data = current_user.netid
    form.name.data = current_user.fullname
    form.employee_type.data = current_user.type
    form.title.data = current_user.descduty
    form.hourly_wage.data = current_user.payrate
    form.account_num.data = current_user.account_num
    form.email.data = current_user.email

