"""Build up the different pages on the site"""

import datetime
from flask_auth import Auth
from flask_login import current_user, login_required, logout_user
from flask import (render_template, request, flash, redirect, make_response, 
                  send_from_directory)
import util
from app import login_manager, app
import client
import forms
import mail
import csv
import user
import statistics
from user import get_user
from util import (PP_FORMAT, DAY_FORMAT, get_start_date, 
                 get_end_date, get_weeks, get_end_date, 
                 get_pay_periods, data_to_datetime)
from forms import (RULoginForm, EmployeeForm, PeriodForm, 
                   HoursForm, DateRangeForm)
from classes import WorkUnit, PayPeriod, Employee

login_manager.login_view = '/login'

@app.route('/')
@login_required
def index():
    """Render index page"""

    # If the user is a new user, force them to update their info
    if not client.user_exists(current_user.netid):
        return redirect("/empinfo", code=307)

    return render_template("index.html",
                           current_user=current_user)


@app.route('/csv')
@login_required
def csv_select_period():
    form = PeriodForm()
    
    start_date = get_start_date_from_request(request, "period")

    warnings = csv.get_warnings(start_date)

    return render_template("generate_csvs.html",
                           form=form,
                           start_date=start_date,
                           warnings=warnings)


@app.route('/csv/download')
@login_required
def generate_csv():
    start_date = get_start_date_from_request(request, "period")

    data = csv.get_csv(start_date)

    # From: http://code.runnable.com/UiIdhKohv5JQAAB6/how-to-download-a-file-generated-on-the-fly-in-flask-for-python
    response = make_response(data)
    response.headers["Content-Disposition"] = "attachment; filename=output.csv"
    return response    


@app.route('/viewhours', methods=["POST", "GET"])
@login_required
def view_hours(this_date=None, link="viewhours.html", user_list=None):
    """Displays a drop down box enabling you to select a pay period.
    Also allows you to see the last four pay periods
    """

    form = PeriodForm()

    # Get start_date from URL argument, or if not specified, set to the current
    # pay period
    start_date = get_start_date_from_request(request, "period")
    end_date = get_end_date(start_date)
    
    weeks = get_weeks(start_date, end_date)

    # Get the payperiod, creating one if it does not exist
    pp = client.get_payperiod(start_date, current_user.netid)
    if not pp:
        pp = client.create_new_payperiod(current_user.netid, start_date)

    # TODO: get data from database to populate table
    # TODO: add client methods to update tables on PeriodForm input
    #       -> makepayperiodfromform, commit, email, etc.
    return render_template(link, form=form, current_user=current_user, 
                           weeks=weeks, pp=pp, start_date=start_date,
                           end_date=end_date.strftime(PP_FORMAT))


@app.route('/edithours', methods=['POST', 'GET'])
@login_required
def edit_hours(link="hoursform.html", user_list=None):
    """Render edit hours page"""
    start_date = get_start_date_from_request(request, "startdate")

    # create form object for editing hours
    hoursform = HoursForm(start_date)

    if hoursform.submit_hours.data and request.method == "POST":
        # Update the db with the value of the "complete" checkbox
        client.mark_payperiod_complete(start_date=start_date,
                    netid=current_user.netid, 
                    is_complete=hoursform.complete.data)

        # Extract time information from hoursform
        for week in hoursform.weeks:
            for day in week.days:
                start_time = day.data['start_time']
                end_time = day.data['end_time']
                start_day = datetime.datetime.strptime(day.data['date_field'], DAY_FORMAT)
                # Convert to datetime objects to be added to database
                if start_time != "--" and end_time != "--":
                    start_time = data_to_datetime(start_day, start_time)
                    end_time = data_to_datetime(start_day, end_time)
                    # TODO: actually put this data in the database
                    client.insert_or_update_work_unit(start_time, end_time,
                                                      start_date, current_user)
                elif start_time == "--" and end_time == "--":
                    client.check_to_delete_work_unit(start_date, start_day, current_user)
                elif start_time == "--":
                    flash("Start time is missing for " + str(start_day.date()))
                else:
                    flash("End time is missing for " + str(start_day.date()))

    pp = client.pay_period_exists(start_date=start_date, netid=current_user.netid)

    if pp:
        # Since the payperiod already exists, set the form to have the value
        # from the database already selected
        pp = client.db_to_pp(pp)

        # First, check the complete box if it's complete
        if pp.complete:
             hoursform.checkmark_complete()

        # Now for hours, we iterate through each day in the form...:
        for week in hoursform.weeks:
            for day in week.days:
                form_date = datetime.datetime.strptime(day.data['date_field'], DAY_FORMAT)
                # Are there any hours in the db for this day?
                for unit in pp.work_units:
                    if form_date.day == unit.start.day:
                        start_time_selection_id = unit.start.strftime("%H%M")
                        end_time_selection_id = unit.end.strftime("%H%M")
                       
                        day.set_start_default(start_time_selection_id)
                        day.set_end_default(end_time_selection_id) 
                        
    else:
        # pp is a payperiod object from classes.py, not models.
        pp = PayPeriod(employee=client.user_to_employee(current_user),
                       start_date=start_date)
        db_pp = client.pp_to_db(pp)
        client.db_add(db_pp)
 
    return render_template(link,
                           hoursform=hoursform,
                           payperiod=pp,
                           user_list=user_list)


@app.route('/empinfo', methods=['GET', 'POST'])
@login_required
def empinfo(new_user_redirected=False):
    """Displays employee form"""

    new_user_redirected = not client.user_exists(current_user.netid)

    if not request.form:
        form = EmployeeForm(request.args)
        forms.prepopulate(form, current_user)
    else:
        form = EmployeeForm(request.form)

    if request.method == 'POST':
        if form.validate():
            user.update_user(form)
        else:
            flash("Failure!")
            app.logger.error("ERROR:\tinvalid input on the"
                             " employee form page.")
            for field_name, error in form.errors.items():
                for msg in error:
                    app.logger.error("ERROR:\tError in signup by {0}[{1}]: {2}"
                                     .format(form.name.data,
                                             form.netid.data,
                                             msg))
                    flash(msg)

    return render_template("empinfo.html",
                           current_user=current_user,
                           form=form,
                           new_user_redirected=new_user_redirected)


@app.route('/summary', methods=["GET", "POST"])
@login_required
def worker_summary(start_date=None, user_list=None):
    """Displays current user's worker summary"""

    # Get list of all netID's
    user_list = client.get_user_list()
    start_date = get_start_date()
    end_date = get_end_date(start_date)
    weeks = get_weeks(start_date, end_date)
    pay_periods = get_pay_periods(start_date, end_date)

    # Render page with new current_user object
    if request.method == 'POST':
        new_user = client.get_user_by_netid(request.form['employee'])
        return render_template('/summary.html',
                               current_user=new_user,
                               pay_periods=pay_periods,
                               user_list=user_list,
                               weeks=weeks)

    # When you render the template, pass in the arguments you determine here.
    return render_template('/summary.html',
                           current_user=current_user,
                           start_date=start_date,
                           user_list=user_list)


@app.route('/stats')
def stats():
    periodform = PeriodForm()
    daterangeform = DateRangeForm()

    start_date = None
    end_date = None

    stats = None
    title = None

    if 'period' in request.args:
        start_date = get_start_date_from_request(request, "period")
        end_date = get_end_date(start_date) 
        title = "the payperiod of {} to {}".format(start_date, end_date)
    elif 'start_date' in request.args and 'end_date' in request.args:
        start_date = datetime.datetime.strptime(request.args['start_date'], 
                                                DAY_FORMAT).date()
        end_date = datetime.datetime.strptime(request.args['end_date'], 
                                              DAY_FORMAT).date()
        title = "{} to {}".format(start_date, end_date)

    if start_date and end_date:
        stats = statistics.get_statistics(start_date, end_date)

    return render_template('/statistics.html',
                           periodform=periodform,
                           daterangeform=daterangeform,
                           stats=stats,
                           title=title)


@app.route('/login', methods=["GET", "POST"])
def login():
    a = Auth(app.config['AUTH'], get_user)
    return a.login(request, RULoginForm, render_login, login_success)


@app.route("/logout")
@login_required
def logout():
    """Handles logging out."""
    logout_user()
    return redirect('/')


# Helper functions

def render_login(**kwargs):
    """ Function to render login template """
    return render_template('login.html', **kwargs)


def login_success(user):
    """ Function to execution on successful login. Redirect to home page """
    return redirect('/')

def get_start_date_from_request(request, arg):
    try:
        start_date = datetime.datetime.\
            strptime(request.args[arg], DAY_FORMAT).date()
        pp_start_date = get_start_date(start_date)
    except KeyError:
        pp_start_date = get_start_date()

    return pp_start_date
